#pragma once
/*
 * Aspera, Inc.
 *
 * C++ only.
 *
 * This is the C++ interface you must implement for sharing a TCP connection
 * with faspstream for tunneling fasp control data (out-of-band feature).
 *
 * You then pass your implementation as an argument to the FaspStreamClient
 * or FaspStreamServer constructors.
 */
#include <exception>

#ifdef WIN32

#ifdef FASPSTREAM_CPP_EXPORTS
#define FASPSTREAM_CPP_API __declspec(dllexport)
#else
#define FASPSTREAM_CPP_API __declspec(dllimport)
#endif

#else // NOT WIN32
#define FASPSTREAM_CPP_API
#endif

/**
 * Abstract interface that internal code will use to share your TCP
 * connection and tunnel fasp control data.
 *

 */
class FaspConnectionChannel
{
public:
    /**
     * Should return number of bytes read from TCP socket.
     *
     * IMPORTANT: Your implementation should be non-blocking / timeout.
     *            For example, you can configure the TCP socket to
     *            timeout after trying to read for so many milliseconds.
     *            If you don't set a timeout or make this non-blocking,
     *            internal code will hang trying to read.
     *
     * Return zero for end of file, negative for error.
     *
     * Throw a FaspChannelTimeoutException if a timeout occurred / there is
     * nothing to read.
     *
     * Throw a FaspChannelClosedException if the TCP socket (unexpectedly)
     * closed.
     */
    virtual int read(char* buff, size_t bytes) = 0;
    /**
     * Should return number of bytes written.
     *
     * write *does not* need to be non-blocking / have a timeout. It can,
     * however, and internal code will continue to retry writing.
     *
     * Return negative on error.
     *
     * Throw FaspChannelClosedException if the TCP socket (unexpectedly)
     * closed.
     */
    virtual int write(const char* data, size_t bytes) = 0;
    /**
     * Internal code calls this when it is finished sharing the TCP
     * connection. You do not need to close the TCP connection you are
     * sharing, but this gives you an opportunity to do so if you wish.
     */
    virtual void close() = 0;
};

/**
 * You should throw this to indicate a read time out.
 */
class FaspChannelTimeoutException : public std::exception
{
};

/**
 * You should throw this to indicate your connection closed
 * (most likely unexpectedly).
 */
class FaspChannelClosedException : public std::exception
{
};

