/*
 * Aspera, Inc.
 *
 * Example client that uses the C wrapper for the FaspStream
 * SDK.
 *
 * Usage:
 *
 *   ./client < outfile
 *
 * Uses mostly default settings. See setup_client.
 */

#include <stdio.h>
#include <stdlib.h>
#include <fasp_stream_client.h>
#include <fasp_stream_constants.h>
#ifdef WIN32
        #include <io.h>
        #include <fcntl.h>
#endif

static
void
log_msg(const char *msg)
{
    fprintf(stderr, "%s\n", msg);
}

static
int
setup_client(fasp_stream_client_t **out)
{
    fasp_stream_client_t *fsc;
    int ret;
    /*
     * Create the client
     */
    ret = fasp_stream_client_new("client-id", &fsc);
    if (ret) {
        log_msg("Error creating client");
        goto fail;
    }
    /*
     * Configure client
     * ----------------
     *
     * Path to faspstream binary (not the SDK library).
     */
    fasp_stream_client_set_faspstream_path(fsc, FASPSTREAM_BINARY);
    /*
     * Set transfer mode to SEND (client will send data to server)
     */
    fasp_stream_client_set_transfer_mode(fsc, 
        FASPSTREAM_TRANSFER_MODE_TYPE_DIRECT_SEND);
    /*
     * Rate target, in Kbps (100 Mbps)
     */
    fasp_stream_client_set_rate_target(fsc, 100000);
    /*
     * Flush internal buffer after 1 second.
     */
    fasp_stream_client_set_force_send_milliseconds(fsc, 1000);
    /*
     * Set port for connecting to server, over TCP
     */
    fasp_stream_client_set_port(fsc, 4444);
    /*
     * Set UDP port for the fasp transfer
     */
    fasp_stream_client_set_udp_port(fsc, 33001);
    /*
     * Register logging function callback
     */
    fasp_stream_client_register_log_callback(fsc, &log_msg);

    *out = fsc;

    return 0;

fail:
    return ret;
}

static
int
read_bytes(char *buff, int len) 
{
    int ret;
    /*
     * Read bytes from stdin
     */
    ret = fread(buff, 1, len, stdin);
    if (ret < 0) {
        log_msg("Trouble reading from stdin?");
        return -1;
    }
    if (ret == 0 && feof(stdin)) {
        return 0;
    }

    return ret;
}

int 
main() {
    fasp_stream_client_t *fsc;
    int ret;
    int count;
    int total_bytes_sent;
    char buff[512];
#ifdef WIN32
    /*
     * Required on Windows for binary files
     */
    _setmode(_fileno(stdin), O_BINARY);
#endif
        /*
     * Create and set up client
     */
    ret = setup_client(&fsc);
    if (ret)
        goto fail1;
    /*
     * Connect to server
     */
    ret = fasp_stream_client_connect(fsc, "localhost");
    if (ret) {
        log_msg("Error connecting to server");
        goto fail2;
    }
    log_msg("Connected to server, sending data...");
    /*
     * Connected to server
     */
    total_bytes_sent = 0;
    for (;;) {
        /*
         * Read bytes from stdin
         */
        count = read_bytes(buff, sizeof(buff));
        if (count <= 0) {
            /*
             * EOF or error
             */
            break;
        }
        /*
         * Write to client's output stream. All bytes are written unless
         * there's an error.
         */
        ret = fasp_stream_client_write(fsc, buff, count);
        if (ret) {
            log_msg("Error writing to client's output stream");
            goto fail2;
        }
        /*
         * Update progress
         */
        total_bytes_sent += count;
        fprintf(stderr, "\rBytes Sent: %d", total_bytes_sent);
        fflush(stderr);
    }
    fprintf(stderr, "\n");
    /*
     * Clean up
     */
    ret = fasp_stream_client_cleanup(fsc);
    if (ret) {
        log_msg("Error cleaning up client");
        goto fail2;
    }

    goto out;

out:
fail2:
    /*
     * Destroy client
     */
    ret = fasp_stream_client_destroy(fsc);
    if (ret) {
        log_msg("Error destroying client");
        abort();
    }
fail1:
    return ret;
}
