/*
 * Aspera, Inc.
 *
 * Example server that uses the C wrapper for the FaspStream
 * SDK with tunneling / out of band fasp control data on one
 * TCP connection.
 *
 * Usage:
 *
 *   ./server > outfile
 *
 * Uses mostly default settings.
 */

#include <stdio.h>
#include <stdlib.h>
#include <fasp_stream_server.h>
#include <fasp_stream_constants.h>
#include <fasp_stream_channel.h>
#include "../poco_stream_socket/poco_stream_socket.h"
#ifdef WIN32
        #include <io.h>
        #include <fcntl.h>
#endif

static
void
err(char *msg)
{
    fprintf(stderr, "error: server: %s\n", msg);
}

static
void
log_msg(const char *msg)
{
    fprintf(stderr, "%s\n", msg);
}

static
int
poco_read(fasp_stream_socket_wrapper_t *w, char *buff, int len, int *timedout)
{
    int ret;
    int sock_ret;
    poco_stream_socket_t *s;
    /*
     * Pull out poco socket from wrapper
     */
    s = (poco_stream_socket_t *)w->priv;
    /*
     * Do read on poco socket
     */
    ret = poco_stream_socket_receive_bytes(s, buff, len, 0, &sock_ret,
        timedout);
    if (ret < 0)
        return ret;

    return sock_ret;
}


static
int
poco_write(fasp_stream_socket_wrapper_t *w, const char *buff, int len)
{
    int ret;
    int sock_ret;
    poco_stream_socket_t *s;
    /*
     * Pull out poco socket from wrapper
     */
    s = (poco_stream_socket_t *)w->priv;
    /*
     * Do write on socket
     */
    ret = poco_stream_socket_send_bytes(s, buff, len, 0, &sock_ret);
    if (ret < 0)
        return ret;

    return sock_ret;
}

static
void
poco_close(fasp_stream_socket_wrapper_t *w)
{
    /*
     * Don't do anything (we'll close it in main).
     */
}

static
int
setup_wrapper(fasp_stream_socket_wrapper_t *sock_wrapper)
{
    int ret;
    poco_stream_socket_t *sock;
    /*
     * Listen for client to connect. (The Poco wrapper just listens
     * for *one* client connection.)
     */
    ret = poco_stream_socket_server(4444, &sock);
    if (ret) {
        err("error listening for client");
        goto fail1;
    }
    /*
     * Set timeout on Poco socket
     */
    poco_stream_socket_set_receive_timeout(sock, FASPSTREAM_CHANNEL_TIMEOUT);
    /*
     * Set up channel
     */
    sock_wrapper->priv = sock;
    sock_wrapper->read = poco_read;
    sock_wrapper->write = poco_write;
    sock_wrapper->close = poco_close;

    return 0;

fail1:
    return ret;
}

static
int
setup_fasp_stream(fasp_stream_socket_wrapper_t *wrapper,
    fasp_stream_server_t **out)
{
    fasp_stream_server_t *fss;
    int ret;
    /*
     * Create the server
     */
    ret = fasp_stream_channel_server_new("server-id", &fss, wrapper);
    if (ret) {
        err("creating server");
        goto fail;
    }
    /*
     * Configure server
     * ----------------
     *
     * Path to faspstream binary (not the SDK library).
     */
    fasp_stream_server_set_faspstream_path(fss, FASPSTREAM_BINARY);
    /*
     * Set transfer mode to RECEIVE (server will receive data from client)
     */
    fasp_stream_server_set_transfer_mode(fss,
        FASPSTREAM_TRANSFER_MODE_TYPE_DIRECT_RECEIVE);
    /*
     * Rate target, in Kbps (100 Mbps)
     */
    fasp_stream_server_set_rate_target(fss, 100000);
    /*
     * Set port to accept TCP connection on
     */
    fasp_stream_server_set_port(fss, 4444);
    /*
     * Set UDP port for the fasp transfer
     */
    fasp_stream_server_set_udp_port(fss, 33001);

    /*
     * Flush internal buffer after 1 second.
     */
    fasp_stream_server_set_force_send_milliseconds(fss, 1000);

    /*
     * Register logging function callback
     */
    fasp_stream_server_register_log_callback(fss, log_msg);

    *out = fss;

    return 0;

fail:
    return ret;
}

static
int
write_bytes(char *buff, int len, int total_bytes_received)
{
    int ret;
    /*
     * Write bytes to stdout
     */
    ret = fwrite(buff, 1, len, stdout);
    if (ret != len) {
        err("trouble writing to stdout?");
        return -1;
    }
    /*
     * Write progress to stderr
     */
    fprintf(stderr, "\rBytes Received: %d", total_bytes_received);
    fflush(stderr);

    return 0;
}

int
main() {
    fasp_stream_server_t *fss;
    fasp_stream_socket_wrapper_t wrapper;

    int ret = 0;
    int count;
    int total_bytes_received;
    char buff[512];

#ifdef WIN32
    /*
     * Required on Windows for binary files.
     */
    _setmode(_fileno(stdout), O_BINARY);
#endif
    /*
     * Listen for client to connect over normal socket, and
     * set up channel for tunneling.
     */
    fprintf(stderr, "Waiting for client to connect...\n");
    ret = setup_wrapper(&wrapper);
    if (ret)
        goto fail0;
    /*
     * Set up faspstream server
     */
    fprintf(stderr, "Client connected, setting up tunneling...\n");
    ret = setup_fasp_stream(&wrapper ,&fss);
    if (ret)
        goto fail1;
    /*
     * Fire up faspstream and tunneling.
     */
    ret = fasp_stream_server_start_server(fss, 0);
    if (ret) {
        err("error starting faspst");
        goto fail2;
    }
    fprintf(stderr, "Tunneling setup, receiving data...\n");
    /*
     * Receive data
     */
    total_bytes_received = 0;
    while ((count = fasp_stream_server_read(fss, buff, sizeof(buff))) > 0) {
        /*
         * Read bytes from client, and write to stdout
         */
        total_bytes_received += count;
        if (write_bytes(buff, count, total_bytes_received))
            break;
    }

    ret = count;

    fprintf(stderr, "\n");

    /*
     * Clean up
     */
    ret = fasp_stream_server_cleanup(fss);
    if (ret) {
        err("error cleaning up server");
        goto fail2;
    }


    goto out;

out:
fail2:
    /*
     * Destroy server
     */
    if (fasp_stream_server_destroy(fss)) {
        err("error destroying server");
        abort();
    }
fail1:
    /*
     * Destroy poco socket
     */
    poco_stream_socket_destroy((poco_stream_socket_t *)wrapper.priv);
fail0:
    return ret;
}
