/*
 * Aspera, Inc.
 *
 * Server.cpp
 *
 * Shows how to listen for an incoming client connection and *send*
 * data from stdin to the client.
 *
 * Usage:
 *
 *    ./Server < in.file
 */
#include <iostream>
#include <fstream>
#include <Constants.h>
#include <FaspOutputStream.h>
#include <FaspStreamServer.h>
#include <FaspStreamException.h>
#ifdef WIN32
        #include <io.h>
        #include <fcntl.h>
#endif

static
void
setupStreams() {
#ifdef WIN32
    /*
     * Required on Windows for binary files
     */
    _setmode(fileno(stdin), O_BINARY);
    _setmode(fileno(stdout), O_BINARY);
#endif
    std::cin.exceptions(std::ifstream::badbit);
    std::cout.exceptions(std::ifstream::badbit);
}

int main()
{
    char data[512];
    setupStreams();
    try {
        /*
         * Initialize faspstream server
         */
        FaspStreamServer fss("Unique-ID-For-a-Client");
        /*
         * Configure server
         * ----------------
         *
         * Set target rate to 100 Mbps
         */
        fss.setRateTarget(100000);
        /*
         * Set the port to connect to
         */
        fss.setPort(4444);
        /*
         * Set the UDP Port for the fasp transfer
         */
        fss.setUdpPort(33001);
        /*
         * Set to *send* mode
         */
        fss.setTransferMode(FASPSTREAM_TRANSFER_MODE_TYPE_DIRECT_SEND);
        /*
         * Start listening for client
         * --------------------------
         */
        std::cerr << "Waiting for client to connect..." << std::endl;
        fss.startServer(true);
        /*
         * Send data
         */
        long totalSent = 0;
        while (std::cin) {
            long sent;
            /*
             * Read from stdin
             */
            std::cin.read(data, sizeof(data));
            sent = std::cin.gcount();
            totalSent += sent;
            /*
             * Write to fasp stream
             */
            fss.getFaspOutputStream().write(data,
                static_cast<int>(sent));

            std::cerr << "Bytes Sent: " << totalSent << "\r";
            std::cerr.flush();
        }
        /*
         * Done; clean up server.
         */
        fss.cleanup();

        std::cerr << std::endl 
                  << "Transfer Complete!" 
                  << std::endl;
        std::cerr << "Bytes Sent: " 
                  << totalSent 
                  << std::endl;
    } 
    catch (std::ofstream::failure &e) {
        std::cerr << "Error reading from stdin: " << std::endl;
        std::cerr << e.what();
    }
    catch (std::exception &e) {
        std::cerr << "A standard exception occurred:" << std::endl;
        std::cerr << e.what();
    }
    catch (...) {
        std::cerr << "An unknown exception ocurred! ";
        std::cerr << "(No further information is available)";
    }

    return 0;
}

